@@ -40,7 +40,7 @@
 //!
 //! \[1\] [parquet-format#nested-encoding](https://github.com/apache/parquet-format#nested-encoding)
 
-use arrow::array::{make_array, Array, ArrayRef, MapArray, StructArray};
+use arrow::array::{make_array, ArrayRef, StructArray};
 use arrow::datatypes::{DataType, Field};
 
 /// Keeps track of the level information per array that is needed to write an Arrow array to Parquet.
@@ -86,11 +86,9 @@ impl LevelType {
     const fn level_increment(&self) -> i16 {
         match self {
             LevelType::Root => 0,
-            // List repetition adds a constant 1
-            LevelType::List(is_nullable) => 1 + *is_nullable as i16,
-            LevelType::Struct(is_nullable) | LevelType::Primitive(is_nullable) => {
-                *is_nullable as i16
-            }
+            LevelType::List(is_nullable)
+            | LevelType::Struct(is_nullable)
+            | LevelType::Primitive(is_nullable) => *is_nullable as i16,
         }
     }
 }
@@ -123,7 +121,7 @@ impl LevelInfo {
     /// The parent struct's nullness is tracked, as it determines whether the child
     /// max_definition should be incremented.
     /// The 'is_parent_struct' variable asks "is this field's parent a struct?".
-    /// * If we are starting at a [RecordBatch](arrow::record_batch::RecordBatch), this is `false`.
+    /// * If we are starting at a [RecordBatch], this is `false`.
     /// * If we are calculating a list's child, this is `false`.
     /// * If we are calculating a struct (i.e. `field.data_type90 == Struct`),
     /// this depends on whether the struct is a child of a struct.
@@ -200,8 +198,9 @@ impl LevelInfo {
                 );
 
                 match child_array.data_type() {
-                    DataType::Null
-                    | DataType::Boolean
+                    // TODO: The behaviour of a <list<null>> is untested
+                    DataType::Null => vec![list_level],
+                    DataType::Boolean
                     | DataType::Int8
                     | DataType::Int16
                     | DataType::Int32
@@ -233,51 +232,11 @@ impl LevelInfo {
                             LevelType::Primitive(list_field.is_nullable()),
                         )]
                     }
-                    DataType::List(_)
-                    | DataType::LargeList(_)
-                    | DataType::Struct(_)
-                    | DataType::Map(_, _) => {
+                    DataType::List(_) | DataType::LargeList(_) | DataType::Struct(_) => {
                         list_level.calculate_array_levels(&child_array, list_field)
                     }
                     DataType::FixedSizeList(_, _) => unimplemented!(),
-                    DataType::Union(_, _) => unimplemented!(),
-                }
-            }
-            DataType::Map(map_field, _) => {
-                // Calculate the map level
-                let map_level = self.calculate_child_levels(
-                    array_offsets,
-                    array_mask,
-                    // A map is treated like a list as it has repetition
-                    LevelType::List(field.is_nullable()),
-                );
-
-                let map_array = array.as_any().downcast_ref::<MapArray>().unwrap();
-
-                let key_array = map_array.keys();
-                let value_array = map_array.values();
-
-                if let DataType::Struct(fields) = map_field.data_type() {
-                    let key_field = &fields[0];
-                    let value_field = &fields[1];
-
-                    let mut map_levels = vec![];
-
-                    // Get key levels
-                    let mut key_levels =
-                        map_level.calculate_array_levels(&key_array, key_field);
-                    map_levels.append(&mut key_levels);
-
-                    let mut value_levels =
-                        map_level.calculate_array_levels(&value_array, value_field);
-                    map_levels.append(&mut value_levels);
-
-                    map_levels
-                } else {
-                    panic!(
-                        "Map field should be a struct, found {:?}",
-                        map_field.data_type()
-                    );
+                    DataType::Union(_) => unimplemented!(),
                 }
             }
             DataType::FixedSizeList(_, _) => unimplemented!(),
@@ -286,18 +245,11 @@ impl LevelInfo {
                     .as_any()
                     .downcast_ref::<StructArray>()
                     .expect("Unable to get struct array");
-                let mut struct_level = self.calculate_child_levels(
+                let struct_level = self.calculate_child_levels(
                     array_offsets,
                     array_mask,
                     LevelType::Struct(field.is_nullable()),
                 );
-
-                // If the parent field is a list, calculate the children of the struct as if it
-                // were a list as well.
-                if matches!(self.level_type, LevelType::List(_)) {
-                    struct_level.level_type = LevelType::List(false);
-                }
-
                 let mut struct_levels = vec![];
                 struct_array
                     .columns()
@@ -310,7 +262,7 @@ impl LevelInfo {
                     });
                 struct_levels
             }
-            DataType::Union(_, _) => unimplemented!(),
+            DataType::Union(_) => unimplemented!(),
             DataType::Dictionary(_, _) => {
                 // Need to check for these cases not implemented in C++:
                 // - "Writing DictionaryArray with nested dictionary type not yet supported"
@@ -382,27 +334,37 @@ impl LevelInfo {
         let mut merged_array_mask = Vec::with_capacity(min_len);
 
         let max_definition = match (self.level_type, level_type) {
-            // Handle the illegal cases
+            (LevelType::Root, LevelType::Struct(is_nullable)) => {
+                // If the struct is non-nullable, its def level doesn't increment
+                is_nullable as i16
+            }
+            (LevelType::Root, _) => 1,
             (_, LevelType::Root) => {
                 unreachable!("Cannot have a root as a child")
             }
+            (LevelType::List(_), _) => {
+                self.max_definition + 1 + level_type.level_increment()
+            }
+            (LevelType::Struct(_), _) => {
+                self.max_definition + level_type.level_increment()
+            }
+            (_, LevelType::List(is_nullable)) => {
+                // if the child is a list, even if its parent is a root
+                self.max_definition + 1 + is_nullable as i16
+            }
             (LevelType::Primitive(_), _) => {
                 unreachable!("Cannot have a primitive parent for any type")
             }
-            // The general case
-            (_, _) => self.max_definition + level_type.level_increment(),
         };
 
         match (self.level_type, level_type) {
             (LevelType::List(_), LevelType::List(is_nullable)) => {
-                // Parent is a list or descendant of a list, and child is a list
+                // parent is a list or descendant of a list, and child is a list
                 let reps = self.repetition.clone().unwrap();
-
-                // List is null, and not empty
-                let l1 = max_definition - is_nullable as i16;
-                // List is not null, but is empty
-                let l2 = max_definition - 1;
-                // List is not null, and not empty
+                // Calculate the 2 list hierarchy definitions in advance
+                // List is not empty, but null
+                let l2 = max_definition - is_nullable as i16;
+                // List is not empty, and not null
                 let l3 = max_definition;
 
                 let mut nulls_seen = 0;
@@ -437,9 +399,7 @@ impl LevelInfo {
                             let merged_mask = parent_mask && child_mask;
 
                             if child_len == 0 {
-                                // Empty slot, i.e. {"parent": {"child": [] } }
-                                // Nullness takes priority over emptiness
-                                definition.push(if child_mask { l2 } else { l1 });
+                                definition.push(parent_def);
                                 repetition.push(parent_rep);
                                 merged_array_mask.push(merged_mask);
                             } else {
@@ -459,7 +419,7 @@ impl LevelInfo {
                                     } else if child_mask {
                                         l3
                                     } else {
-                                        l1
+                                        l2
                                     });
                                     repetition.push(rep);
                                     merged_array_mask.push(merged_mask);
@@ -546,11 +506,9 @@ impl LevelInfo {
                 // Encountering a list for the first time.
                 // Calculate the 2 list hierarchy definitions in advance
 
-                // List is null, and not empty
-                let l1 = max_definition - 1 - is_nullable as i16;
-                // List is not null, but is empty
-                let l2 = max_definition - 1;
-                // List is not null, and not empty
+                // List is not empty, but null (if nullable)
+                let l2 = max_definition - is_nullable as i16;
+                // List is not empty, and not null
                 let l3 = max_definition;
 
                 self.definition
@@ -565,24 +523,20 @@ impl LevelInfo {
 
                         match (parent_mask, child_len) {
                             (true, 0) => {
-                                // Empty slot, i.e. {"parent": {"child": [] } }
-                                // Nullness takes priority over emptiness
-                                definition.push(if child_mask { l2 } else { l1 });
+                                // empty slot that is valid, i.e. {"parent": {"child": [] } }
+                                definition.push(if child_mask { l3 } else { l2 });
                                 repetition.push(0);
                                 merged_array_mask.push(child_mask);
                             }
                             (false, 0) => {
-                                // Inherit the parent definition as parent was null
                                 definition.push(*def);
                                 repetition.push(0);
                                 merged_array_mask.push(child_mask);
                             }
                             (true, _) => {
                                 (child_from..child_to).for_each(|child_index| {
-                                    // l1 and l3 make sense as list is not empty,
-                                    // but we reflect that it's either null or not
-                                    definition.push(if child_mask { l3 } else { l1 });
-                                    // Mark the first child slot as 0, and the next as 1
+                                    definition.push(if child_mask { l3 } else { l2 });
+                                    // mark the first child slot as 0, and the next as 1
                                     repetition.push(if child_index == child_from {
                                         0
                                     } else {
@@ -593,7 +547,6 @@ impl LevelInfo {
                             }
                             (false, _) => {
                                 (child_from..child_to).for_each(|child_index| {
-                                    // Inherit the parent definition as parent was null
                                     definition.push(*def);
                                     // mark the first child slot as 0, and the next as 1
                                     repetition.push(if child_index == child_from {
@@ -676,9 +629,8 @@ impl LevelInfo {
         len: usize,
     ) -> (Vec<i64>, Vec<bool>) {
         match array.data_type() {
-            // A NullArray is entirely nulls, despite not containing a null buffer
-            DataType::Null => ((0..=(len as i64)).collect(), vec![false; len]),
-            DataType::Boolean
+            DataType::Null
+            | DataType::Boolean
             | DataType::Int8
             | DataType::Int16
             | DataType::Int32
@@ -710,12 +662,13 @@ impl LevelInfo {
                 };
                 ((0..=(len as i64)).collect(), array_mask)
             }
-            DataType::List(_) | DataType::Map(_, _) => {
-                let offsets = unsafe { array.data().buffers()[0].typed_data::<i32>() };
+            DataType::List(_) => {
+                let data = array.data();
+                let offsets = unsafe { data.buffers()[0].typed_data::<i32>() };
                 let offsets = offsets
-                    .iter()
-                    .copied()
-                    .skip(array.offset() + offset)
+                    .to_vec()
+                    .into_iter()
+                    .skip(offset)
                     .take(len + 1)
                     .map(|v| v as i64)
                     .collect::<Vec<i64>>();
@@ -728,7 +681,7 @@ impl LevelInfo {
             DataType::LargeList(_) => {
                 let offsets = unsafe { array.data().buffers()[0].typed_data::<i64>() }
                     .iter()
-                    .skip(array.offset() + offset)
+                    .skip(offset)
                     .take(len + 1)
                     .copied()
                     .collect();
@@ -749,7 +702,7 @@ impl LevelInfo {
                     array_mask,
                 )
             }
-            DataType::FixedSizeList(_, _) | DataType::Union(_, _) => {
+            DataType::FixedSizeList(_, _) | DataType::Union(_) => {
                 unimplemented!("Getting offsets not yet implemented")
             }
         }
@@ -757,14 +710,14 @@ impl LevelInfo {
 
     /// Given a level's information, calculate the offsets required to index an array correctly.
     pub(crate) fn filter_array_indices(&self) -> Vec<usize> {
-        if !matches!(self.level_type, LevelType::Primitive(_)) {
-            panic!(
+        // happy path if not dealing with lists
+        let is_nullable = match self.level_type {
+            LevelType::Primitive(is_nullable) => is_nullable,
+            _ => panic!(
                 "Cannot filter indices on a non-primitive array, found {:?}",
                 self.level_type
-            );
-        }
-
-        // happy path if not dealing with lists
+            ),
+        };
         if self.repetition.is_none() {
             return self
                 .definition
@@ -779,26 +732,17 @@ impl LevelInfo {
                 })
                 .collect();
         }
-
         let mut filtered = vec![];
-        let mut definition_levels = self.definition.iter();
+        // remove slots that are false from definition_mask
         let mut index = 0;
-
-        for len in self.array_offsets.windows(2).map(|s| s[1] - s[0]) {
-            if len == 0 {
-                // Skip this definition level--the iterator should not be empty, and the definition
-                // level be less than max_definition, i.e., a null value)
-                assert!(*definition_levels.next().unwrap() < self.max_definition);
-            } else {
-                for (_, def) in (0..len).zip(&mut definition_levels) {
-                    if *def == self.max_definition {
-                        filtered.push(index);
-                    }
-                    index += 1;
-                }
+        self.definition.iter().for_each(|def| {
+            if *def == self.max_definition {
+                filtered.push(index);
             }
-        }
-
+            if *def >= self.max_definition - is_nullable as i16 {
+                index += 1;
+            }
+        });
         filtered
     }
 }
@@ -923,12 +867,11 @@ mod tests {
             LevelType::Primitive(false),
         );
         let expected_levels = LevelInfo {
-            // As it is non-null, definitions can be omitted
-            definition: vec![0; 10],
+            definition: vec![1; 10],
             repetition: None,
             array_offsets,
             array_mask,
-            max_definition: 0,
+            max_definition: 1,
             level_type: LevelType::Primitive(false),
             offset: 0,
             length: 10,
@@ -1005,13 +948,13 @@ mod tests {
             // - Calculate the level at the list
             // - Calculate the level at the list's child
             // We do not do this in these tests, thus the levels are 1 less.
-            definition: vec![2, 2, 0, 2, 2, 2, 2, 2, 2, 2, 2, 2],
+            definition: vec![1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1],
             repetition: Some(vec![0, 1, 0, 0, 1, 0, 1, 1, 1, 0, 1, 1]),
             array_offsets,
             array_mask: vec![
                 true, true, false, true, true, true, true, true, true, true, true, true,
             ],
-            max_definition: 2,
+            max_definition: 1,
             level_type: LevelType::List(true),
             offset: 0,
             length: 11, // the child has 11 slots
@@ -1063,14 +1006,14 @@ mod tests {
             // 2 3 [4] are 0
             // 4 5 6 7 [8] are 1 (defined at level 1 only)
             // 8 9 10 [11] are 2 (defined at both levels)
-            definition: vec![0, 0, 1, 0, 0, 3, 3, 3, 3, 3, 3, 3],
+            definition: vec![0, 0, 1, 0, 0, 2, 2, 2, 2, 2, 2, 2],
             repetition: Some(vec![0, 1, 0, 0, 1, 0, 1, 1, 1, 0, 1, 1]),
             array_offsets,
             array_mask: vec![
                 false, false, false, false, false, true, true, true, true, true, true,
                 true,
             ],
-            max_definition: 3,
+            max_definition: 2,
             level_type: LevelType::List(true),
             offset: 0,
             length: 11,
@@ -1118,7 +1061,7 @@ mod tests {
             // 3: [[108, 109], [110, 111], [112, 113], [114, 115]]
             // 4: [[116, 117], [118, 119], [120, 121]]
             definition: vec![
-                0, 0, 0, 0, 1, 0, 0, 0, 0, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
+                0, 0, 0, 0, 1, 0, 0, 0, 0, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4,
             ],
             repetition: Some(vec![
                 0, 2, 1, 2, 0, 0, 2, 1, 2, 0, 2, 1, 2, 1, 2, 1, 2, 0, 2, 1, 2, 1, 2,
@@ -1129,7 +1072,7 @@ mod tests {
                 true, true, true, true, true, true, true, true, true, true, true, true,
                 true,
             ],
-            max_definition: 5,
+            max_definition: 4,
             level_type: LevelType::List(true),
             offset: 0,
             length: 22,
@@ -1178,11 +1121,11 @@ mod tests {
         // 2: [4, 5]
         // 3: [6, 7]
         let expected_levels = LevelInfo {
-            definition: vec![1, 3, 3, 3, 3, 3, 3, 3],
+            definition: vec![1, 2, 2, 2, 2, 2, 2, 2],
             repetition: Some(vec![0, 0, 1, 1, 0, 1, 0, 1]),
             array_offsets,
             array_mask: vec![false, true, true, true, true, true, true, true],
-            max_definition: 3,
+            max_definition: 2,
             level_type: LevelType::List(true),
             offset: 0,
             length: 8,
@@ -1224,14 +1167,14 @@ mod tests {
         // 2: {"struct": [ [204, 205, 206], [207, 208, 209, 210] ]}
         // 3: {"struct": [ [], [211, 212, 213, 214, 215] ]}
         let expected_levels = LevelInfo {
-            definition: vec![1, 5, 5, 5, 4, 5, 5, 5, 5, 5, 5, 5, 4, 5, 5, 5, 5, 5],
+            definition: vec![1, 4, 4, 4, 2, 4, 4, 4, 4, 4, 4, 4, 2, 4, 4, 4, 4, 4],
             repetition: Some(vec![0, 0, 1, 2, 1, 0, 2, 2, 1, 2, 2, 2, 0, 1, 2, 2, 2, 2]),
             array_mask: vec![
                 false, true, true, true, false, true, true, true, true, true, true, true,
                 true, true, true, true, true, true,
             ],
             array_offsets,
-            max_definition: 5,
+            max_definition: 4,
             level_type: LevelType::List(true),
             offset: 0,
             length: 16,
@@ -1319,8 +1262,7 @@ mod tests {
             .add_buffer(a_value_offsets)
             .null_bit_buffer(Buffer::from(vec![0b00011011]))
             .add_child_data(a_values.data().clone())
-            .build()
-            .unwrap();
+            .build();
 
         assert_eq!(a_list_data.null_count(), 1);
 
@@ -1423,8 +1365,7 @@ mod tests {
             .len(5)
             .add_buffer(g_value_offsets)
             .add_child_data(g_value.data().clone())
-            .build()
-            .unwrap();
+            .build();
         let g = ListArray::from(g_list_data);
 
         let e = StructArray::from(vec![
@@ -1475,11 +1416,11 @@ mod tests {
         let list_level = levels.get(0).unwrap();
 
         let expected_level = LevelInfo {
-            definition: vec![0, 0, 0, 0, 0],
+            definition: vec![1, 1, 1, 1, 1],
             repetition: None,
             array_offsets: vec![0, 1, 2, 3, 4, 5],
             array_mask: vec![true, true, true, true, true],
-            max_definition: 0,
+            max_definition: 1,
             level_type: LevelType::Primitive(false),
             offset: 0,
             length: 5,
@@ -1604,219 +1545,4 @@ mod tests {
             panic!("Levels should not be equal, to reflect the difference in struct nullness");
         }
     }
-
-    #[test]
-    fn test_map_array() {
-        // Note: we are using the JSON Arrow reader for brevity
-        let json_content = r#"
-        {"stocks":{"long": "$AAA", "short": "$BBB"}}
-        {"stocks":{"long": null, "long": "$CCC", "short": null}}
-        {"stocks":{"hedged": "$YYY", "long": null, "short": "$D"}}
-        "#;
-        let entries_struct_type = DataType::Struct(vec![
-            Field::new("key", DataType::Utf8, false),
-            Field::new("value", DataType::Utf8, true),
-        ]);
-        let stocks_field = Field::new(
-            "stocks",
-            DataType::Map(
-                Box::new(Field::new("entries", entries_struct_type, false)),
-                false,
-            ),
-            // not nullable, so the keys have max level = 1
-            false,
-        );
-        let schema = Arc::new(Schema::new(vec![stocks_field]));
-        let builder = arrow::json::ReaderBuilder::new()
-            .with_schema(schema)
-            .with_batch_size(64);
-        let mut reader = builder.build(std::io::Cursor::new(json_content)).unwrap();
-
-        let batch = reader.next().unwrap().unwrap();
-
-        let expected_batch_level = LevelInfo {
-            definition: vec![0; 3],
-            repetition: None,
-            array_offsets: (0..=3).collect(),
-            array_mask: vec![true, true, true],
-            max_definition: 0,
-            level_type: LevelType::Root,
-            offset: 0,
-            length: 3,
-        };
-
-        let batch_level = LevelInfo::new(0, 3);
-        assert_eq!(&batch_level, &expected_batch_level);
-
-        // calculate the map's level
-        let mut levels = vec![];
-        batch
-            .columns()
-            .iter()
-            .zip(batch.schema().fields())
-            .for_each(|(array, field)| {
-                let mut array_levels = batch_level.calculate_array_levels(array, field);
-                levels.append(&mut array_levels);
-            });
-        assert_eq!(levels.len(), 2);
-
-        // test key levels
-        let list_level = levels.get(0).unwrap();
-
-        let expected_level = LevelInfo {
-            definition: vec![1; 7],
-            repetition: Some(vec![0, 1, 0, 1, 0, 1, 1]),
-            array_offsets: vec![0, 2, 4, 7],
-            array_mask: vec![true; 7],
-            max_definition: 1,
-            level_type: LevelType::Primitive(false),
-            offset: 0,
-            length: 7,
-        };
-        assert_eq!(list_level, &expected_level);
-
-        // test values levels
-        let list_level = levels.get(1).unwrap();
-
-        let expected_level = LevelInfo {
-            definition: vec![2, 2, 2, 1, 2, 1, 2],
-            repetition: Some(vec![0, 1, 0, 1, 0, 1, 1]),
-            array_offsets: vec![0, 2, 4, 7],
-            array_mask: vec![true, true, true, false, true, false, true],
-            max_definition: 2,
-            level_type: LevelType::Primitive(true),
-            offset: 0,
-            length: 7,
-        };
-        assert_eq!(list_level, &expected_level);
-    }
-
-    #[test]
-    fn test_list_of_struct() {
-        // define schema
-        let int_field = Field::new("a", DataType::Int32, true);
-        let item_field =
-            Field::new("item", DataType::Struct(vec![int_field.clone()]), true);
-        let list_field = Field::new("list", DataType::List(Box::new(item_field)), true);
-
-        let int_builder = Int32Builder::new(10);
-        let struct_builder =
-            StructBuilder::new(vec![int_field], vec![Box::new(int_builder)]);
-        let mut list_builder = ListBuilder::new(struct_builder);
-
-        // [{a: 1}], [], null, [null, null], [{a: null}], [{a: 2}]
-        //
-        // [{a: 1}]
-        let values = list_builder.values();
-        values
-            .field_builder::<Int32Builder>(0)
-            .unwrap()
-            .append_value(1)
-            .unwrap();
-        values.append(true).unwrap();
-        list_builder.append(true).unwrap();
-
-        // []
-        list_builder.append(true).unwrap();
-
-        // null
-        list_builder.append(false).unwrap();
-
-        // [null, null]
-        let values = list_builder.values();
-        values
-            .field_builder::<Int32Builder>(0)
-            .unwrap()
-            .append_null()
-            .unwrap();
-        values.append(false).unwrap();
-        values
-            .field_builder::<Int32Builder>(0)
-            .unwrap()
-            .append_null()
-            .unwrap();
-        values.append(false).unwrap();
-        list_builder.append(true).unwrap();
-
-        // [{a: null}]
-        let values = list_builder.values();
-        values
-            .field_builder::<Int32Builder>(0)
-            .unwrap()
-            .append_null()
-            .unwrap();
-        values.append(true).unwrap();
-        list_builder.append(true).unwrap();
-
-        // [{a: 2}]
-        let values = list_builder.values();
-        values
-            .field_builder::<Int32Builder>(0)
-            .unwrap()
-            .append_value(2)
-            .unwrap();
-        values.append(true).unwrap();
-        list_builder.append(true).unwrap();
-
-        let array = Arc::new(list_builder.finish());
-
-        let schema = Arc::new(Schema::new(vec![list_field]));
-
-        let rb = RecordBatch::try_new(schema, vec![array]).unwrap();
-
-        let batch_level = LevelInfo::new(0, rb.num_rows());
-        let list_level =
-            &batch_level.calculate_array_levels(rb.column(0), rb.schema().field(0))[0];
-
-        let expected_level = LevelInfo {
-            definition: vec![4, 1, 0, 2, 2, 3, 4],
-            repetition: Some(vec![0, 0, 0, 0, 1, 0, 0]),
-            array_offsets: vec![0, 1, 1, 1, 3, 4, 5],
-            array_mask: vec![true, true, false, false, false, false, true],
-            max_definition: 4,
-            level_type: LevelType::Primitive(true),
-            offset: 0,
-            length: 5,
-        };
-
-        assert_eq!(list_level, &expected_level);
-    }
-
-    #[test]
-    fn test_nested_indices() {
-        // Given a buffer like
-        // [0, null, null, 1, 2]
-        //
-        // The two level infos below might represent the two structures
-        // 1: [{a: 0}], [], null, [null, null], [{a: 1}], [{a: 2}]
-        // 2: [0], [], null, [null, null], [1], [2]
-        //
-        // (That is, their only difference is that the leaf values are nested one level deeper in a
-        // struct).
-
-        let level1 = LevelInfo {
-            definition: vec![4, 1, 0, 2, 2, 4, 4],
-            repetition: Some(vec![0, 0, 0, 0, 1, 0, 0]),
-            array_offsets: vec![0, 1, 1, 1, 3, 4, 5],
-            array_mask: vec![true, true, false, false, false, false, true],
-            max_definition: 4,
-            level_type: LevelType::Primitive(true),
-            offset: 0,
-            length: 5,
-        };
-
-        let level2 = LevelInfo {
-            definition: vec![3, 1, 0, 2, 2, 3, 3],
-            repetition: Some(vec![0, 0, 0, 0, 1, 0, 0]),
-            array_offsets: vec![0, 1, 1, 1, 3, 4, 5],
-            array_mask: vec![true, true, false, false, false, false, true],
-            max_definition: 3,
-            level_type: LevelType::Primitive(true),
-            offset: 0,
-            length: 5,
-        };
-
-        // filter_array_indices should return the same indices in this case.
-        assert_eq!(level1.filter_array_indices(), level2.filter_array_indices());
-    }
 }
