@@ -16,113 +16,75 @@
 // under the License.
 
 use std::cmp::{max, min};
+use std::mem::{replace, size_of};
 
-use arrow::bitmap::Bitmap;
-use arrow::buffer::Buffer;
-
-use crate::arrow::record_reader::{
-    buffer::{BufferQueue, ScalarBuffer, ValuesBuffer},
-    definition_levels::{DefinitionLevelBuffer, DefinitionLevelDecoder},
-};
-use crate::column::{
-    page::PageReader,
-    reader::{
-        decoder::{ColumnLevelDecoderImpl, ColumnValueDecoder, ColumnValueDecoderImpl},
-        GenericColumnReader,
-    },
-};
+use crate::column::{page::PageReader, reader::ColumnReaderImpl};
 use crate::data_type::DataType;
 use crate::errors::{ParquetError, Result};
 use crate::schema::types::ColumnDescPtr;
-
-pub(crate) mod buffer;
-mod definition_levels;
+use arrow::array::BooleanBufferBuilder;
+use arrow::bitmap::Bitmap;
+use arrow::buffer::{Buffer, MutableBuffer};
 
 const MIN_BATCH_SIZE: usize = 1024;
 
 /// A `RecordReader` is a stateful column reader that delimits semantic records.
-pub type RecordReader<T> =
-    GenericRecordReader<ScalarBuffer<<T as DataType>::T>, ColumnValueDecoderImpl<T>>;
-
-#[doc(hidden)]
-/// A generic stateful column reader that delimits semantic records
-///
-/// This type is hidden from the docs, and relies on private traits with no
-/// public implementations. As such this type signature may be changed without
-/// breaking downstream users as it can only be constructed through type aliases
-pub struct GenericRecordReader<V, CV> {
+pub struct RecordReader<T: DataType> {
     column_desc: ColumnDescPtr,
 
-    records: V,
-    def_levels: Option<DefinitionLevelBuffer>,
-    rep_levels: Option<ScalarBuffer<i16>>,
-    column_reader:
-        Option<GenericColumnReader<ColumnLevelDecoderImpl, DefinitionLevelDecoder, CV>>,
+    records: MutableBuffer,
+    def_levels: Option<MutableBuffer>,
+    rep_levels: Option<MutableBuffer>,
+    null_bitmap: Option<BooleanBufferBuilder>,
+    column_reader: Option<ColumnReaderImpl<T>>,
 
     /// Number of records accumulated in records
     num_records: usize,
     /// Number of values `num_records` contains.
     num_values: usize,
 
+    values_seen: usize,
     /// Starts from 1, number of values have been written to buffer
     values_written: usize,
+    in_middle_of_record: bool,
 }
 
-impl<V, CV> GenericRecordReader<V, CV>
-where
-    V: ValuesBuffer + Default,
-    CV: ColumnValueDecoder<Slice = V::Slice>,
-{
-    /// Create a new [`GenericRecordReader`]
-    pub fn new(desc: ColumnDescPtr) -> Self {
-        Self::new_with_options(desc, false)
-    }
+impl<T: DataType> RecordReader<T> {
+    pub fn new(column_schema: ColumnDescPtr) -> Self {
+        let (def_levels, null_map) = if column_schema.max_def_level() > 0 {
+            (
+                Some(MutableBuffer::new(MIN_BATCH_SIZE)),
+                Some(BooleanBufferBuilder::new(0)),
+            )
+        } else {
+            (None, None)
+        };
 
-    /// Create a new [`GenericRecordReader`] with the ability to only generate the bitmask
-    ///
-    /// If `null_mask_only` is true only the null bitmask will be generated and
-    /// [`Self::consume_def_levels`] and [`Self::consume_rep_levels`] will always return `None`
-    ///
-    /// It is insufficient to solely check that that the max definition level is 1 as we
-    /// need there to be no nullable parent array that will required decoded definition levels
-    ///
-    /// In particular consider the case of:
-    ///
-    /// ```ignore
-    /// message nested {
-    ///   OPTIONAL Group group {
-    ///     REQUIRED INT32 leaf;
-    ///   }
-    /// }
-    /// ```
-    ///
-    /// The maximum definition level of leaf is 1, however, we still need to decode the
-    /// definition levels so that the parent group can be constructed correctly
-    ///
-    pub(crate) fn new_with_options(desc: ColumnDescPtr, null_mask_only: bool) -> Self {
-        let def_levels = (desc.max_def_level() > 0)
-            .then(|| DefinitionLevelBuffer::new(&desc, null_mask_only));
-
-        let rep_levels = (desc.max_rep_level() > 0).then(ScalarBuffer::new);
+        let rep_levels = if column_schema.max_rep_level() > 0 {
+            Some(MutableBuffer::new(MIN_BATCH_SIZE))
+        } else {
+            None
+        };
 
         Self {
-            records: Default::default(),
+            records: MutableBuffer::new(MIN_BATCH_SIZE),
             def_levels,
             rep_levels,
+            null_bitmap: null_map,
             column_reader: None,
-            column_desc: desc,
+            column_desc: column_schema,
             num_records: 0,
             num_values: 0,
+            values_seen: 0,
             values_written: 0,
+            in_middle_of_record: false,
         }
     }
 
     /// Set the current page reader.
     pub fn set_page_reader(&mut self, page_reader: Box<dyn PageReader>) -> Result<()> {
-        self.column_reader = Some(GenericColumnReader::new(
-            self.column_desc.clone(),
-            page_reader,
-        ));
+        self.column_reader =
+            Some(ColumnReaderImpl::new(self.column_desc.clone(), page_reader));
         Ok(())
     }
 
@@ -145,52 +107,25 @@ where
         loop {
             // Try to find some records from buffers that has been read into memory
             // but not counted as seen records.
-            let (record_count, value_count) =
-                self.count_records(num_records - records_read);
+            records_read += self.split_records(num_records - records_read)?;
 
-            self.num_records += record_count;
-            self.num_values += value_count;
-            records_read += record_count;
+            // Since page reader contains complete records, so if we reached end of a
+            // page reader, we should reach the end of a record
+            if end_of_column
+                && self.values_seen >= self.values_written
+                && self.in_middle_of_record
+            {
+                self.num_records += 1;
+                self.num_values = self.values_seen;
+                self.in_middle_of_record = false;
+                records_read += 1;
+            }
 
-            if records_read == num_records {
+            if (records_read >= num_records) || end_of_column {
                 break;
             }
 
-            if end_of_column {
-                // Since page reader contains complete records, if we reached end of a
-                // page reader, we should reach the end of a record
-                if self.rep_levels.is_some() {
-                    self.num_records += 1;
-                    self.num_values = self.values_written;
-                    records_read += 1;
-                }
-                break;
-            }
-
-            // If repetition levels present, we don't know how much more to read
-            // in order to read the requested number of records, therefore read at least
-            // MIN_BATCH_SIZE, otherwise read **exactly** what was requested. This helps
-            // to avoid a degenerate case where the buffers are never fully drained.
-            //
-            // Consider the scenario where the user is requesting batches of MIN_BATCH_SIZE.
-            //
-            // When transitioning across a row group boundary, this will read some remainder
-            // from the row group `r`, before reading MIN_BATCH_SIZE from the next row group,
-            // leaving `MIN_BATCH_SIZE + r` in the buffer.
-            //
-            // The client will then only split off the `MIN_BATCH_SIZE` they actually wanted,
-            // leaving behind `r`. This will continue indefinitely.
-            //
-            // Aside from wasting cycles splitting and shuffling buffers unnecessarily, this
-            // prevents dictionary preservation from functioning correctly as the buffer
-            // will never be emptied, allowing a new dictionary to be registered.
-            //
-            // This degenerate case can still occur for repeated fields, but
-            // it is avoided for the more common case of a non-repeated field
-            let batch_size = match &self.rep_levels {
-                Some(_) => max(num_records - records_read, MIN_BATCH_SIZE),
-                None => num_records - records_read,
-            };
+            let batch_size = max(num_records - records_read, MIN_BATCH_SIZE);
 
             // Try to more value from parquet pages
             let values_read = self.read_one_batch(batch_size)?;
@@ -219,31 +154,108 @@ where
     /// definition level values that have already been read into memory but not counted
     /// as record values, e.g. those from `self.num_values` to `self.values_written`.
     pub fn consume_def_levels(&mut self) -> Result<Option<Buffer>> {
-        Ok(match self.def_levels.as_mut() {
-            Some(x) => x.split_levels(self.num_values),
-            None => None,
-        })
+        let new_buffer = if let Some(ref mut def_levels_buf) = &mut self.def_levels {
+            let num_left_values = self.values_written - self.num_values;
+            // create an empty buffer, as it will be resized below
+            let mut new_buffer = MutableBuffer::new(0);
+            let num_bytes = num_left_values * size_of::<i16>();
+            let new_len = self.num_values * size_of::<i16>();
+
+            new_buffer.resize(num_bytes, 0);
+
+            let new_def_levels = new_buffer.as_slice_mut();
+            let left_def_levels = &def_levels_buf.as_slice_mut()[new_len..];
+
+            new_def_levels[0..num_bytes].copy_from_slice(&left_def_levels[0..num_bytes]);
+
+            def_levels_buf.resize(new_len, 0);
+            Some(new_buffer)
+        } else {
+            None
+        };
+
+        Ok(replace(&mut self.def_levels, new_buffer).map(|x| x.into()))
     }
 
     /// Return repetition level data.
     /// The side effect is similar to `consume_def_levels`.
     pub fn consume_rep_levels(&mut self) -> Result<Option<Buffer>> {
-        Ok(match self.rep_levels.as_mut() {
-            Some(x) => Some(x.split_off(self.num_values)),
-            None => None,
-        })
+        // TODO: Optimize to reduce the copy
+        let new_buffer = if let Some(ref mut rep_levels_buf) = &mut self.rep_levels {
+            let num_left_values = self.values_written - self.num_values;
+            // create an empty buffer, as it will be resized below
+            let mut new_buffer = MutableBuffer::new(0);
+            let num_bytes = num_left_values * size_of::<i16>();
+            let new_len = self.num_values * size_of::<i16>();
+
+            new_buffer.resize(num_bytes, 0);
+
+            let new_rep_levels = new_buffer.as_slice_mut();
+            let left_rep_levels = &rep_levels_buf.as_slice_mut()[new_len..];
+
+            new_rep_levels[0..num_bytes].copy_from_slice(&left_rep_levels[0..num_bytes]);
+
+            rep_levels_buf.resize(new_len, 0);
+
+            Some(new_buffer)
+        } else {
+            None
+        };
+
+        Ok(replace(&mut self.rep_levels, new_buffer).map(|x| x.into()))
     }
 
     /// Returns currently stored buffer data.
     /// The side effect is similar to `consume_def_levels`.
-    pub fn consume_record_data(&mut self) -> Result<V::Output> {
-        Ok(self.records.split_off(self.num_values))
+    pub fn consume_record_data(&mut self) -> Result<Buffer> {
+        // TODO: Optimize to reduce the copy
+        let num_left_values = self.values_written - self.num_values;
+        // create an empty buffer, as it will be resized below
+        let mut new_buffer = MutableBuffer::new(0);
+        let num_bytes = num_left_values * T::get_type_size();
+        let new_len = self.num_values * T::get_type_size();
+
+        new_buffer.resize(num_bytes, 0);
+
+        let new_records = new_buffer.as_slice_mut();
+        let left_records = &mut self.records.as_slice_mut()[new_len..];
+
+        new_records[0..num_bytes].copy_from_slice(&left_records[0..num_bytes]);
+
+        self.records.resize(new_len, 0);
+
+        Ok(replace(&mut self.records, new_buffer).into())
     }
 
     /// Returns currently stored null bitmap data.
     /// The side effect is similar to `consume_def_levels`.
     pub fn consume_bitmap_buffer(&mut self) -> Result<Option<Buffer>> {
-        Ok(self.consume_bitmap()?.map(|b| b.into_buffer()))
+        // TODO: Optimize to reduce the copy
+        if self.column_desc.max_def_level() > 0 {
+            assert!(self.null_bitmap.is_some());
+            let num_left_values = self.values_written - self.num_values;
+            let new_bitmap_builder = Some(BooleanBufferBuilder::new(max(
+                MIN_BATCH_SIZE,
+                num_left_values,
+            )));
+
+            let old_bitmap = replace(&mut self.null_bitmap, new_bitmap_builder)
+                .map(|mut builder| builder.finish())
+                .unwrap();
+
+            let old_bitmap = Bitmap::from(old_bitmap);
+
+            for i in self.num_values..self.values_written {
+                self.null_bitmap
+                    .as_mut()
+                    .unwrap()
+                    .append(old_bitmap.is_set(i));
+            }
+
+            Ok(Some(old_bitmap.into_buffer()))
+        } else {
+            Ok(None)
+        }
     }
 
     /// Reset state of record reader.
@@ -253,26 +265,49 @@ where
         self.values_written -= self.num_values;
         self.num_records = 0;
         self.num_values = 0;
+        self.values_seen = 0;
+        self.in_middle_of_record = false;
     }
 
     /// Returns bitmap data.
     pub fn consume_bitmap(&mut self) -> Result<Option<Bitmap>> {
-        Ok(self
-            .def_levels
-            .as_mut()
-            .map(|levels| levels.split_bitmask(self.num_values)))
+        self.consume_bitmap_buffer()
+            .map(|buffer| buffer.map(Bitmap::from))
     }
 
     /// Try to read one batch of data.
     fn read_one_batch(&mut self, batch_size: usize) -> Result<usize> {
-        let rep_levels = self
-            .rep_levels
-            .as_mut()
-            .map(|levels| levels.spare_capacity_mut(batch_size));
+        // Reserve spaces
+        self.records
+            .resize(self.records.len() + batch_size * T::get_type_size(), 0);
+        if let Some(ref mut buf) = self.rep_levels {
+            buf.resize(buf.len() + batch_size * size_of::<i16>(), 0);
+        }
+        if let Some(ref mut buf) = self.def_levels {
+            buf.resize(buf.len() + batch_size * size_of::<i16>(), 0);
+        }
 
-        let def_levels = self.def_levels.as_mut();
+        let values_written = self.values_written;
 
-        let values = self.records.spare_capacity_mut(batch_size);
+        // Convert mutable buffer spaces to mutable slices
+        let (prefix, values, suffix) =
+            unsafe { self.records.as_slice_mut().align_to_mut::<T::T>() };
+        assert!(prefix.is_empty() && suffix.is_empty());
+        let values = &mut values[values_written..];
+
+        let def_levels = self.def_levels.as_mut().map(|buf| {
+            let (prefix, def_levels, suffix) =
+                unsafe { buf.as_slice_mut().align_to_mut::<i16>() };
+            assert!(prefix.is_empty() && suffix.is_empty());
+            &mut def_levels[values_written..]
+        });
+
+        let rep_levels = self.rep_levels.as_mut().map(|buf| {
+            let (prefix, rep_levels, suffix) =
+                unsafe { buf.as_slice_mut().align_to_mut::<i16>() };
+            assert!(prefix.is_empty() && suffix.is_empty());
+            &mut rep_levels[values_written..]
+        });
 
         let (values_read, levels_read) = self
             .column_reader
@@ -280,61 +315,98 @@ where
             .unwrap()
             .read_batch(batch_size, def_levels, rep_levels, values)?;
 
+        // get new references for the def levels.
+        let def_levels = self.def_levels.as_ref().map(|buf| {
+            let (prefix, def_levels, suffix) =
+                unsafe { buf.as_slice().align_to::<i16>() };
+            assert!(prefix.is_empty() && suffix.is_empty());
+            &def_levels[values_written..]
+        });
+
+        let max_def_level = self.column_desc.max_def_level();
+
         if values_read < levels_read {
-            let def_levels = self.def_levels.as_ref().ok_or_else(|| {
+            let def_levels = def_levels.ok_or_else(|| {
                 general_err!(
                     "Definition levels should exist when data is less than levels!"
                 )
             })?;
 
-            self.records.pad_nulls(
-                self.values_written,
-                values_read,
-                levels_read,
-                def_levels.nulls().as_slice(),
-            );
+            // Fill spaces in column data with default values
+            let mut values_pos = values_read;
+            let mut level_pos = levels_read;
+
+            while level_pos > values_pos {
+                if def_levels[level_pos - 1] == max_def_level {
+                    // This values is not empty
+                    // We use swap rather than assign here because T::T doesn't
+                    // implement Copy
+                    values.swap(level_pos - 1, values_pos - 1);
+                    values_pos -= 1;
+                } else {
+                    values[level_pos - 1] = T::T::default();
+                }
+
+                level_pos -= 1;
+            }
         }
 
-        let values_read = max(levels_read, values_read);
+        // Fill in bitmap data
+        if let Some(null_buffer) = self.null_bitmap.as_mut() {
+            let def_levels = def_levels.ok_or_else(|| {
+                general_err!(
+                    "Definition levels should exist when data is less than levels!"
+                )
+            })?;
+            (0..levels_read)
+                .for_each(|idx| null_buffer.append(def_levels[idx] == max_def_level));
+        }
+
+        let values_read = max(values_read, levels_read);
         self.set_values_written(self.values_written + values_read)?;
         Ok(values_read)
     }
 
-    /// Inspects the buffered repetition levels in the range `self.num_values..self.values_written`
-    /// and returns the number of "complete" records along with the corresponding number of values
-    ///
-    /// A "complete" record is one where the buffer contains a subsequent repetition level of 0
-    fn count_records(&self, records_to_read: usize) -> (usize, usize) {
-        match self.rep_levels.as_ref() {
+    /// Split values into records according repetition definition and returns number of
+    /// records read.
+    #[allow(clippy::unnecessary_wraps)]
+    fn split_records(&mut self, records_to_read: usize) -> Result<usize> {
+        let rep_levels = self.rep_levels.as_ref().map(|buf| {
+            let (prefix, rep_levels, suffix) =
+                unsafe { buf.as_slice().align_to::<i16>() };
+            assert!(prefix.is_empty() && suffix.is_empty());
+            rep_levels
+        });
+
+        match rep_levels {
             Some(buf) => {
-                let buf = buf.as_slice();
-
                 let mut records_read = 0;
-                let mut end_of_last_record = self.num_values;
 
-                for (current, item) in buf
-                    .iter()
-                    .enumerate()
-                    .take(self.values_written)
-                    .skip(self.num_values)
+                while (self.values_seen < self.values_written)
+                    && (records_read < records_to_read)
                 {
-                    if *item == 0 && current != self.num_values {
-                        records_read += 1;
-                        end_of_last_record = current;
-
-                        if records_read == records_to_read {
-                            break;
+                    if buf[self.values_seen] == 0 {
+                        if self.in_middle_of_record {
+                            records_read += 1;
+                            self.num_records += 1;
+                            self.num_values = self.values_seen;
                         }
+                        self.in_middle_of_record = true;
                     }
+                    self.values_seen += 1;
                 }
 
-                (records_read, end_of_last_record - self.num_values)
+                Ok(records_read)
             }
             None => {
                 let records_read =
-                    min(records_to_read, self.values_written - self.num_values);
+                    min(records_to_read, self.values_written - self.values_seen);
+                self.num_records += records_read;
+                self.num_values += records_read;
+                self.values_seen += records_read;
+                self.in_middle_of_record = false;
 
-                (records_read, records_read)
+                Ok(records_read)
             }
         }
     }
@@ -342,14 +414,17 @@ where
     #[allow(clippy::unnecessary_wraps)]
     fn set_values_written(&mut self, new_values_written: usize) -> Result<()> {
         self.values_written = new_values_written;
-        self.records.set_len(self.values_written);
+        self.records
+            .resize(self.values_written * T::get_type_size(), 0);
+
+        let new_levels_len = self.values_written * size_of::<i16>();
 
         if let Some(ref mut buf) = self.rep_levels {
-            buf.set_len(self.values_written)
+            buf.resize(new_levels_len, 0)
         };
 
         if let Some(ref mut buf) = self.def_levels {
-            buf.set_len(self.values_written)
+            buf.resize(new_levels_len, 0)
         };
 
         Ok(())
@@ -358,12 +433,7 @@ where
 
 #[cfg(test)]
 mod tests {
-    use std::sync::Arc;
-
-    use arrow::array::{Int16BufferBuilder, Int32BufferBuilder};
-    use arrow::bitmap::Bitmap;
-    use arrow::buffer::Buffer;
-
+    use super::RecordReader;
     use crate::basic::Encoding;
     use crate::column::page::Page;
     use crate::column::page::PageReader;
@@ -372,11 +442,12 @@ mod tests {
     use crate::schema::parser::parse_message_type;
     use crate::schema::types::SchemaDescriptor;
     use crate::util::test_common::page_util::{DataPageBuilder, DataPageBuilderImpl};
-
-    use super::RecordReader;
+    use arrow::array::{BooleanBufferBuilder, Int16BufferBuilder, Int32BufferBuilder};
+    use arrow::bitmap::Bitmap;
+    use std::sync::Arc;
 
     struct TestPageReader {
-        pages: Box<dyn Iterator<Item = Page> + Send>,
+        pages: Box<dyn Iterator<Item = Page>>,
     }
 
     impl TestPageReader {
@@ -393,14 +464,6 @@ mod tests {
         }
     }
 
-    impl Iterator for TestPageReader {
-        type Item = Result<Page>;
-
-        fn next(&mut self) -> Option<Self::Item> {
-            self.get_next_page().transpose()
-        }
-    }
-
     #[test]
     fn test_read_required_records() {
         // Construct column schema
@@ -553,6 +616,15 @@ mod tests {
             assert_eq!(7, record_reader.num_values());
         }
 
+        // Verify result record data
+        let mut bb = Int32BufferBuilder::new(7);
+        bb.append_slice(&[0, 7, 0, 6, 3, 0, 8]);
+        let expected_buffer = bb.finish();
+        assert_eq!(
+            expected_buffer,
+            record_reader.consume_record_data().unwrap()
+        );
+
         // Verify result def levels
         let mut bb = Int16BufferBuilder::new(7);
         bb.append_slice(&[1i16, 2i16, 0i16, 2i16, 2i16, 0i16, 2i16]);
@@ -563,28 +635,13 @@ mod tests {
         );
 
         // Verify bitmap
-        let expected_valid = &[false, true, false, true, true, false, true];
-        let expected_buffer = Buffer::from_iter(expected_valid.iter().cloned());
-        let expected_bitmap = Bitmap::from(expected_buffer);
+        let mut bb = BooleanBufferBuilder::new(7);
+        bb.append_slice(&[false, true, false, true, true, false, true]);
+        let expected_bitmap = Bitmap::from(bb.finish());
         assert_eq!(
             Some(expected_bitmap),
             record_reader.consume_bitmap().unwrap()
         );
-
-        // Verify result record data
-        let actual = record_reader.consume_record_data().unwrap();
-        let actual_values = unsafe { actual.typed_data::<i32>() };
-
-        let expected = &[0, 7, 0, 6, 3, 0, 8];
-        assert_eq!(actual_values.len(), expected.len());
-
-        // Only validate valid values are equal
-        let iter = expected_valid.iter().zip(actual_values).zip(expected);
-        for ((valid, actual), expected) in iter {
-            if *valid {
-                assert_eq!(actual, expected)
-            }
-        }
     }
 
     #[test]
@@ -667,6 +724,15 @@ mod tests {
             assert_eq!(9, record_reader.num_values());
         }
 
+        // Verify result record data
+        let mut bb = Int32BufferBuilder::new(9);
+        bb.append_slice(&[4, 0, 0, 7, 6, 3, 2, 8, 9]);
+        let expected_buffer = bb.finish();
+        assert_eq!(
+            expected_buffer,
+            record_reader.consume_record_data().unwrap()
+        );
+
         // Verify result def levels
         let mut bb = Int16BufferBuilder::new(9);
         bb.append_slice(&[2i16, 0i16, 1i16, 2i16, 2i16, 2i16, 2i16, 2i16, 2i16]);
@@ -677,27 +743,13 @@ mod tests {
         );
 
         // Verify bitmap
-        let expected_valid = &[true, false, false, true, true, true, true, true, true];
-        let expected_buffer = Buffer::from_iter(expected_valid.iter().cloned());
-        let expected_bitmap = Bitmap::from(expected_buffer);
+        let mut bb = BooleanBufferBuilder::new(9);
+        bb.append_slice(&[true, false, false, true, true, true, true, true, true]);
+        let expected_bitmap = Bitmap::from(bb.finish());
         assert_eq!(
             Some(expected_bitmap),
             record_reader.consume_bitmap().unwrap()
         );
-
-        // Verify result record data
-        let actual = record_reader.consume_record_data().unwrap();
-        let actual_values = unsafe { actual.typed_data::<i32>() };
-        let expected = &[4, 0, 0, 7, 6, 3, 2, 8, 9];
-        assert_eq!(actual_values.len(), expected.len());
-
-        // Only validate valid values are equal
-        let iter = expected_valid.iter().zip(actual_values).zip(expected);
-        for ((valid, actual), expected) in iter {
-            if *valid {
-                assert_eq!(actual, expected)
-            }
-        }
     }
 
     #[test]
